SystemOrganization addCategory: #FuelPreview!

FLAnalysis subclass: #FLPreviewAnalysis
	instanceVariableNames: 'traceForPreview currentlyTraced references'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPreview'!

!FLPreviewAnalysis commentStamp: 'MartinDias 12/24/2011 03:57' prior: 0!
I am an analysis specialized for doing a visualization of the graph being to serialize.

Examples of use:

(FLAnalyzer newDefault 
	showPreview 
	analysisFor: {1. {{3. {4. {5}}}. 2}}) 
	openPathsTo: 5;
	open.
		
(FLAnalyzer newDefault 
	showPreview 
	analysisFor: (SortedCollection sortBlock: [:a :b | a > b ])) 
	openPathsTo: #DoIt;
	open.
	
(FLAnalyzer newDefault 
	showPreview;
	considerInternalAll: { FLPerson };
	analysisFor: FLPerson) 
	openPathsTo: #=;
	open.
	!

!FLPreviewAnalysis methodsFor: 'initialization' stamp: 'MartinDias 12/24/2011 03:15'!
initialize
	super initialize.
	references := IdentitySet new.! !

!FLPreviewAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 12/24/2011 02:12'!
mapAndTrace: anObject

	| aCluster |
	aCluster := self clusterFor: anObject.
	
	traceForPreview := false. "don't add to view the cluster-level references"
	clusterization add: aCluster traceWith: self.

	traceForPreview := true.
	currentlyTraced := anObject.
	aCluster add: anObject traceWith: self.
! !

!FLPreviewAnalysis methodsFor: 'visualizing' stamp: 'MartinDias 12/24/2011 03:56'!
open

	FLPreviewBuilder new 
		mark: root with: Color red;
		references: references;
		open! !

!FLPreviewAnalysis methodsFor: 'visualizing' stamp: 'MartinDias 12/24/2011 03:55'!
openPathsTo: anObject

	FLPreviewBuilder new 
		mark: root with: Color red;
		mark: anObject with: Color yellow;
		references: (self referencesTo: anObject);
		open! !

!FLPreviewAnalysis methodsFor: 'accessing' stamp: 'MartinDias 12/24/2011 03:51'!
referencesTo: anObject

	^self referencesTo: anObject excluding: #()! !

!FLPreviewAnalysis methodsFor: 'private' stamp: 'MartinDias 12/24/2011 03:51'!
referencesTo: anObject excluding: excludedObjects

	| referencesToAnObject result |
	(excludedObjects includes: anObject)
		ifTrue: [^Array new].
	referencesToAnObject := references select: [:association | anObject == association value ].

	result := referencesToAnObject.
	referencesToAnObject do: [:association |
		association key == root
			ifFalse: [result addAll: (self referencesTo: association key excluding: excludedObjects, {anObject})]].
	^result! !

!FLPreviewAnalysis methodsFor: 'accessing' stamp: 'MartinDias 12/24/2011 02:14'!
referencesToAll
	^ references! !

!FLPreviewAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 12/24/2011 02:13'!
trace: aReferencedObject

	super trace: aReferencedObject.
	traceForPreview ifTrue: [
		references add: (currentlyTraced -> aReferencedObject)]! !

!FLAnalyzer methodsFor: '*FuelPreview' stamp: 'MartinDias 12/4/2011 01:27'!
setPreviewAnalysis

	analysisFactory := [:anObject | 
		(FLPreviewAnalysis 
			newWith: self firstInMapperChain 
			root: anObject)
			run;
			yourself ]! !

!FLAnalyzer methodsFor: '*FuelPreview' stamp: 'MartinDias 12/4/2011 01:28'!
showPreview

	self setPreviewAnalysis! !

Object subclass: #FLPreviewBuilder
	instanceVariableNames: 'references elementByObjectMap colors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPreview'!

!FLPreviewBuilder methodsFor: 'private' stamp: 'MartinDias 12/24/2011 03:36'!
colorFor: anObject

	^ colors
		at: anObject
		ifAbsent: [Color black]
! !

!FLPreviewBuilder methodsFor: 'private' stamp: 'MartinDias 12/24/2011 02:44'!
edgeFor: anAssociation
	^ (ROEdge from: (self nodeFor: anAssociation key) to: (self nodeFor: anAssociation value))
		addDecorator: (ROLine new color: Color green);
		yourself.! !

!FLPreviewBuilder methodsFor: 'initialize-release' stamp: 'MartinDias 12/24/2011 03:37'!
initialize

	super initialize.
	elementByObjectMap := IdentityDictionary new.
	colors := IdentityDictionary new.! !

!FLPreviewBuilder methodsFor: 'accessing' stamp: 'MartinDias 12/24/2011 03:39'!
mark: anObject with: aColor 
	
	colors at: anObject put: aColor! !

!FLPreviewBuilder methodsFor: 'private' stamp: 'MartinDias 12/24/2011 02:07'!
newNodeFor: anObject
	^ (ROElement on: anObject)
			extent: 15@15;
			addDecorator: (ROCircle new color: (self colorFor: anObject));
			addInteraction: RODraggable;
			addInteraction: (ROPopup text: #printString);
			on: ROMouseRightClick do: [ :e | anObject inspect ];
			yourself! !

!FLPreviewBuilder methodsFor: 'private' stamp: 'MartinDias 12/24/2011 02:46'!
nodeFor: anObject

	^ elementByObjectMap 
		at: anObject 
		ifAbsentPut: (self newNodeFor: anObject)! !

!FLPreviewBuilder methodsFor: 'opening' stamp: 'MartinDias 12/24/2011 02:54'!
open
	"Example of use:
	| root |
	root := {1. {{3. {4. {5}}}. 2}}.
	FLPreviewBuilder new 
		root: root;
		references: ((FLAnalyzer newDefault showPreview analysisFor: root) referencesTo: 5);
		open
	"
	| view nodes edges |
	edges := references collect: [:association | self edgeFor: association ].
	nodes := elementByObjectMap values.

	view := ROView new.
	view addAll: edges.
	view addAll: nodes.
	
	ROTreeLayout on: nodes.
	view @RODraggable @ RODraggableWithVelocity.
	view open.! !

!FLPreviewBuilder methodsFor: 'accessing' stamp: 'MartinDias 12/24/2011 02:30'!
references
	^ references! !

!FLPreviewBuilder methodsFor: 'accessing' stamp: 'MartinDias 12/24/2011 02:30'!
references: anObject
	references := anObject! !

TestCase subclass: #FLPreviewAnalysisTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPreview'!

!FLPreviewAnalysisTest methodsFor: 'accessing' stamp: 'MartinDias 12/24/2011 00:12'!
analysisOf: anObject

	^ FLAnalyzer newDefault 
		showPreview; 
		analysisFor: anObject
	! !

!FLPreviewAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 12/24/2011 00:10'!
testAllReferencesInNestedArray
	
	| array references expectedReferences |
	array := #(1 #(42)).
	references := (self analysisOf: array) referencesToAll.
	expectedReferences := Set 
		with: #(1 #(42)) -> 1 
		with: #(1 #(42)) -> #(42) 
		with: #(42) -> 42.
	
	self assert: references = expectedReferences.
! !

!FLPreviewAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 12/24/2011 00:10'!
testOnePathInNestedArray
	
	| array references expectedReferences |
	array := #(1 #(42)).
	references := (self analysisOf: array) referencesTo: 42.
	expectedReferences := Set 
		with: #(1 #(42)) -> #(42) 
		with: #(42) -> 42.
	
	self assert: references = expectedReferences.
! !

!FLPreviewAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 12/24/2011 03:33'!
testOnePathInNestedArrayWithRepetition
	
	| array references expectedReferences subArray |
	subArray := Array with: 42.
	array := Array with: subArray with: subArray.
	references := (self analysisOf: array) referencesTo: 42.
	expectedReferences := Set 
		with: array -> subArray 
		with: subArray -> 42.
	
	expectedReferences do: [:expectedAssociation |
		self assert: (references anySatisfy: [:association | 
			association key == expectedAssociation key and: [association value == expectedAssociation value]])].! !

!FLPreviewAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 12/24/2011 00:11'!
testTwoPathsInNestedArray
	
	| array references expectedReferences |
	array := #(42 #(42)).
	references := (self analysisOf: array) referencesTo: 42.
	expectedReferences := Set 
		with: #(42 #(42)) -> 42 
		with: #(42 #(42)) -> #(42) 
		with: #(42) -> 42.
	
	self assert: references = expectedReferences.
! !
